$(function() {
	$('[data-parallax]').parallax();
});

/**
   * [@todo add ability to attach multiple handlers to a single element]
   *
   * eg. single:  <div data-parallax="prop:'top'" />
   *     multi :  <div data-parallax="[ {prop:'top', yScale:0.5}, {prop:'left', yScale: 0.8} ]" />
   *
   * [@todo calculate screen limits of each element to avoid recalcs]
   *
   * [@todo add shortcut for translate3d/2d to help avoid reflows]
   */
(function($) {
	var useLimiting = false;

	var defaultOptions = {
		yScale: 0.5,
		yOrigin: 0, // offset in pixels from original placement
		yMin: undefined,
		yMax: undefined,
		prop: 'background-position-y',
		units: 'px'
	};

	var manager;

	function _defer(fn, self) {
		setTimeout(fn.bind(self), 0);
	}

	function _limit(freq, fn, self) {
		var timerHandle;
		return function() {
			if (!timerHandle) {
				timerHandle = setTimeout(function() {
					timerHandle = undefined;
				}, freq);
				return fn.call(self, arguments);
			}
			return;
		};
	}

	function ParallaxHandler($el, opts) {
		this.opts = $.extend(
			{},
			defaultOptions,
			{
				$el: $el,
				yOrigin: $el.offset().top
			},
			opts
		);
	}

	$.extend(ParallaxHandler.prototype, {
		update: function(offset) {
			var newValue = (offset - this.opts.yOrigin) * this.opts.yScale;
			if (this.opts.yMin && newValue < this.opts.yMin) {
				newValue = this.opts.yMin;
			}
			if (this.opts.yMax && newValue > this.opts.yMax) {
				newValue = this.opts.yMax;
			}

			//      console.log(this, this.opts.prop, ':', offset - newValue + this.opts.units );

			this.opts.$el.css(this.opts.prop, newValue + this.opts.units);
		}
	});

	function ParallaxManager() {
		this.handlers = [];
		// us limit to prevent many duplicate events
		var fn = useLimiting ? _limit(2, this.onScroll, this) : this.onScroll.bind(this);

		$(window).on('scroll', fn);
	}

	$.extend(ParallaxManager.prototype, {
		makeHandler: function($el, opts) {
			var handler = new ParallaxHandler($el, opts);
			this.handlers.push(handler);
			return handler;
		},
		onScroll: function(e) {
			_defer(function() {
				var scrollTop = $(window).scrollTop() + $(window).height() / 2;
				//console.log($(window).scrollTop());
				this.handlers.forEach(function(handler, idx) {
					handler.update(scrollTop);
				});
			}, this);
		}
	});

	$.fn.parallax = function(opts) {
		if (!manager) {
			manager = new ParallaxManager();
		}
		this.each(function(idx, item) {
			var dataOpts = new Function('return {' + $(item).data('parallax') + '}')();
			dataOpts = $.extend({}, opts, dataOpts);
			var handler = manager.makeHandler($(item), dataOpts);
		});
	};
})(jQuery);
